@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import os
 import sys
+from contextlib import suppress
 from copy import deepcopy
 from datetime import datetime, timedelta
 from functools import reduce
@@ -64,6 +65,9 @@ if TYPE_CHECKING:
     from semantic_release.commit_parser.conventional import (
         ConventionalCommitParser,
     )
+    from semantic_release.commit_parser.conventional.parser_monorepo import (
+        ConventionalCommitMonorepoParser,
+    )
     from semantic_release.commit_parser.emoji import EmojiCommitParser
     from semantic_release.commit_parser.scipy import ScipyCommitParser
     from semantic_release.commit_parser.token import ParsedMessageResult, ParseResult
@@ -73,6 +77,7 @@ if TYPE_CHECKING:
         GetParserFromConfigFileFn,
         UpdateVersionPyFileFn,
     )
+    from tests.fixtures.monorepos.git_monorepo import BuildMonorepoFn
 
     try:
         # Python 3.8 and 3.9 compatibility
@@ -155,6 +160,7 @@ if TYPE_CHECKING:
             extra_configs: dict[str, TomlSerializableTypes] | None = None,
             mask_initial_release: bool = True,  # Default as of v10
             package_name: str = ...,
+            monorepo: bool = False,
         ) -> tuple[Path, HvcsBase]: ...
 
     class CommitNReturnChangelogEntryFn(Protocol):
@@ -301,6 +307,34 @@ if TYPE_CHECKING:
         mask_initial_release: bool
         extra_configs: dict[str, TomlSerializableTypes]
 
+    class RepoActionConfigureMonorepo(TypedDict):
+        action: Literal[RepoActionStep.CONFIGURE_MONOREPO]
+        details: RepoActionConfigureMonorepoDetails
+
+    class RepoActionConfigureMonorepoDetails(DetailsBase):
+        package_dir: Path | str
+        package_name: str
+        tag_format_str: str | None
+        mask_initial_release: bool
+        extra_configs: dict[str, TomlSerializableTypes]
+
+    class RepoActionCreateMonorepo(TypedDict):
+        action: Literal[RepoActionStep.CREATE_MONOREPO]
+        details: RepoActionCreateMonorepoDetails
+
+    class RepoActionCreateMonorepoDetails(DetailsBase):
+        commit_type: CommitConvention
+        hvcs_client_name: str
+        hvcs_domain: str
+        origin_url: NotRequired[str]
+
+    class RepoActionChangeDirectory(TypedDict):
+        action: Literal[RepoActionStep.CHANGE_DIRECTORY]
+        details: RepoActionChangeDirectoryDetails
+
+    class RepoActionChangeDirectoryDetails(DetailsBase):
+        directory: Path | str
+
     class RepoActionMakeCommits(TypedDict):
         action: Literal[RepoActionStep.MAKE_COMMITS]
         details: RepoActionMakeCommitsDetails
@@ -381,6 +415,7 @@ if TYPE_CHECKING:
             commit_spec: CommitSpec,
             commit_type: CommitConvention,
             parser: CommitParser[ParseResult, ParserOptions],
+            monorepo: bool = ...,
         ) -> CommitDef: ...
 
     class GetRepoDefinitionFn(Protocol):
@@ -415,6 +450,7 @@ if TYPE_CHECKING:
             commits: Sequence[CommitSpec],
             commit_type: CommitConvention,
             parser: CommitParser[ParseResult, ParserOptions],
+            monorepo: bool = ...,
         ) -> Sequence[CommitDef]: ...
 
     class BuildSpecificRepoFn(Protocol):
@@ -423,7 +459,10 @@ if TYPE_CHECKING:
         ) -> Sequence[RepoActions]: ...
 
     RepoActions: TypeAlias = Union[
+        RepoActionChangeDirectory,
         RepoActionConfigure,
+        RepoActionConfigureMonorepo,
+        RepoActionCreateMonorepo,
         RepoActionGitCheckout,
         RepoActionGitMerge[RepoActionGitMergeDetails],
         RepoActionGitMerge[RepoActionGitFFMergeDetails],
@@ -528,15 +567,28 @@ def cached_example_git_project(
         # the implementation on Windows holds some file descriptors open until close is called.
         with Repo.init(cached_repo_path) as repo:
             rmtree(str(Path(repo.git_dir, "hooks")))
+
+            # set up remote origin (including a refs directory)
+            git_origin_dir = Path(repo.common_dir, "refs", "remotes", "origin")
+            repo.create_remote(name=git_origin_dir.name, url=example_git_https_url)
+            git_origin_dir.mkdir(parents=True, exist_ok=True)
+
             # Without this the global config may set it to "master", we want consistency
             repo.git.branch("-M", DEFAULT_BRANCH_NAME)
+
             with repo.config_writer("repository") as config:
                 config.set_value("user", "name", commit_author.name)
                 config.set_value("user", "email", commit_author.email)
                 config.set_value("commit", "gpgsign", False)
                 config.set_value("tag", "gpgsign", False)
 
-            repo.create_remote(name="origin", url=example_git_https_url)
+                # set up a remote tracking branch for the default branch
+                config.set_value(f'branch "{DEFAULT_BRANCH_NAME}"', "remote", "origin")
+                config.set_value(
+                    f'branch "{DEFAULT_BRANCH_NAME}"',
+                    "merge",
+                    f"refs/heads/{DEFAULT_BRANCH_NAME}",
+                )
 
             # make sure all base files are in index to enable initial commit
             repo.index.add(("*", ".gitignore"))
@@ -611,6 +663,43 @@ def get_commit_def_of_conventional_commit() -> GetCommitDefFn[ConventionalCommit
     return _get_commit_def
 
 
+@pytest.fixture(scope="session")
+def get_commit_def_of_conventional_commit_monorepo() -> (
+    GetCommitDefFn[ConventionalCommitMonorepoParser]
+):
+    def _get_commit_def(
+        msg: str, parser: ConventionalCommitMonorepoParser
+    ) -> CommitDef:
+        if not (parsed_result := parser.parse_message(msg)):
+            return {
+                "cid": "",
+                "msg": msg,
+                "type": "unknown",
+                "category": "Unknown",
+                "desc": msg,
+                "brking_desc": "",
+                "scope": "",
+                "mr": "",
+                "sha": NULL_HEX_SHA,
+                "include_in_changelog": False,
+            }
+
+        return {
+            "cid": "",
+            "msg": msg,
+            "type": parsed_result.type,
+            "category": parsed_result.category,
+            "desc": str.join("\n\n", parsed_result.descriptions),
+            "brking_desc": str.join("\n\n", parsed_result.breaking_descriptions),
+            "scope": parsed_result.scope,
+            "mr": parsed_result.linked_merge_request,
+            "sha": NULL_HEX_SHA,
+            "include_in_changelog": True,
+        }
+
+    return _get_commit_def
+
+
 @pytest.fixture(scope="session")
 def get_commit_def_of_emoji_commit() -> GetCommitDefFn[EmojiCommitParser]:
     def _get_commit_def_of_emoji_commit(
@@ -906,6 +995,15 @@ def create_squash_merge_commit(
             date=commit_dt.isoformat(timespec="seconds"),
         )
 
+        # After a merge we need to ensure the remote tracking branch is updated
+        with suppress(TypeError):
+            # Fake an automated push to remote by updating the remote tracking branch
+            # Detached HEAD commits won't have an active branch and throw a TypeError
+            git_repo.git.update_ref(
+                f"refs/remotes/origin/{git_repo.active_branch.name}",
+                git_repo.head.commit.hexsha,
+            )
+
         # return the commit definition with the sha & message updated
         return {
             **commit_def,
@@ -955,6 +1053,14 @@ def create_release_tagged_commit(
             date=commit_dt.isoformat(timespec="seconds"),
         )
 
+        with suppress(TypeError):
+            # Fake an automated push to remote by updating the remote tracking branch
+            # Detached HEAD commits won't have an active branch and throw a TypeError
+            git_repo.git.update_ref(
+                f"refs/remotes/origin/{git_repo.active_branch.name}",
+                git_repo.head.commit.hexsha,
+            )
+
         # ensure commit timestamps are unique (adding one second even though a nanosecond has gone by)
         commit_dt += timedelta(seconds=1)
 
@@ -992,6 +1098,14 @@ def commit_n_rtn_changelog_entry(
             date=commit_dt.isoformat(timespec="seconds"),
         )
 
+        with suppress(TypeError):
+            # Fake an automated push to remote by updating the remote tracking branch
+            # Detached HEAD commits won't have an active branch and throw a TypeError
+            git_repo.git.update_ref(
+                f"refs/remotes/origin/{git_repo.active_branch.name}",
+                git_repo.head.commit.hexsha,
+            )
+
         # Capture the resulting commit message and sha
         return {
             **commit_def,
@@ -1078,6 +1192,7 @@ def build_configured_base_repo(  # noqa: C901
         extra_configs: dict[str, TomlSerializableTypes] | None = None,
         mask_initial_release: bool = True,  # Default as of v10
         package_name: str = EXAMPLE_PROJECT_NAME,
+        monorepo: bool = False,
     ) -> tuple[Path, HvcsBase]:
         if not cached_example_git_project.exists():
             raise RuntimeError("Unable to find cached git project files!")
@@ -1094,6 +1209,7 @@ def build_configured_base_repo(  # noqa: C901
             extra_configs=extra_configs,
             mask_initial_release=mask_initial_release,
             package_name=package_name,
+            monorepo=monorepo,
         )
 
     return _build_configured_base_repo
@@ -1131,16 +1247,19 @@ def configure_base_repo(  # noqa: C901
         extra_configs: dict[str, TomlSerializableTypes] | None = None,
         mask_initial_release: bool = True,  # Default as of v10
         package_name: str = EXAMPLE_PROJECT_NAME,
+        monorepo: bool = False,
     ) -> tuple[Path, HvcsBase]:
         # Make sure we are in the dest directory
         with temporary_working_directory(dest_dir):
             # Set parser configuration
             if commit_type == "conventional":
-                use_conventional_parser(toml_file=pyproject_toml_file)
+                use_conventional_parser(
+                    toml_file=pyproject_toml_file, monorepo=monorepo
+                )
             elif commit_type == "emoji":
-                use_emoji_parser(toml_file=pyproject_toml_file)
+                use_emoji_parser(toml_file=pyproject_toml_file, monorepo=monorepo)
             elif commit_type == "scipy":
-                use_scipy_parser(toml_file=pyproject_toml_file)
+                use_scipy_parser(toml_file=pyproject_toml_file, monorepo=monorepo)
             else:
                 use_custom_parser(commit_type, toml_file=pyproject_toml_file)
 
@@ -1290,12 +1409,16 @@ def separate_squashed_commit_def() -> SeparateSquashedCommitDefFn:
 @pytest.fixture(scope="session")
 def convert_commit_spec_to_commit_def(
     get_commit_def_of_conventional_commit: GetCommitDefFn[ConventionalCommitParser],
+    get_commit_def_of_conventional_commit_monorepo: GetCommitDefFn[
+        ConventionalCommitMonorepoParser
+    ],
     get_commit_def_of_emoji_commit: GetCommitDefFn[EmojiCommitParser],
     get_commit_def_of_scipy_commit: GetCommitDefFn[ScipyCommitParser],
     stable_now_date: datetime,
 ) -> ConvertCommitSpecToCommitDefFn:
     message_parsers = {
         "conventional": get_commit_def_of_conventional_commit,
+        "conventional-monorepo": get_commit_def_of_conventional_commit_monorepo,
         "emoji": get_commit_def_of_emoji_commit,
         "scipy": get_commit_def_of_scipy_commit,
     }
@@ -1304,8 +1427,12 @@ def convert_commit_spec_to_commit_def(
         commit_spec: CommitSpec,
         commit_type: CommitConvention,
         parser: CommitParser[ParseResult, ParserOptions],
+        monorepo: bool = False,
     ) -> CommitDef:
-        parse_msg_fn = cast("GetCommitDefFn[Any]", message_parsers[commit_type])
+        parse_msg_fn = cast(
+            "GetCommitDefFn[Any]",
+            message_parsers[f"{commit_type}{'-monorepo' if monorepo else ''}"],
+        )
 
         # Extract the correct commit message for the commit type
         return {
@@ -1330,9 +1457,12 @@ def convert_commit_specs_to_commit_defs(
         commits: Sequence[CommitSpec],
         commit_type: CommitConvention,
         parser: CommitParser[ParseResult, ParserOptions],
+        monorepo: bool = False,
     ) -> Sequence[CommitDef]:
         return [
-            convert_commit_spec_to_commit_def(commit, commit_type, parser=parser)
+            convert_commit_spec_to_commit_def(
+                commit, commit_type, parser=parser, monorepo=monorepo
+            )
             for commit in commits
         ]
 
@@ -1342,6 +1472,8 @@ def convert_commit_specs_to_commit_defs(
 @pytest.fixture(scope="session")
 def build_repo_from_definition(  # noqa: C901, its required and its just test code
     build_configured_base_repo: BuildRepoFn,
+    build_base_monorepo: BuildMonorepoFn,
+    configure_monorepo_package: BuildRepoFn,
     default_tag_format_str: str,
     create_release_tagged_commit: CreateReleaseFn,
     create_squash_merge_commit: CreateSquashMergeCommitFn,
@@ -1386,6 +1518,7 @@ def build_repo_from_definition(  # noqa: C901, its required and its just test co
         )
 
         repo_dir = Path(dest_dir).resolve().absolute()
+        commit_type: CommitConvention = "conventional"
         hvcs: Github | Gitlab | Gitea | Bitbucket
         commit_cache: dict[str, CommitDef] = {}
         current_repo_def: dict[Version | Literal["Unreleased"], RepoVersionDef] = {}
@@ -1418,6 +1551,64 @@ def build_repo_from_definition(  # noqa: C901, its required and its just test co
                         },
                     )
 
+                elif action == RepoActionStep.CREATE_MONOREPO:
+                    cfg_mr_def = cast(
+                        "RepoActionCreateMonorepoDetails", step_result["details"]
+                    )
+                    build_base_monorepo(dest_dir=repo_dir)
+                    hvcs = get_hvcs(
+                        hvcs_client_name=cfg_mr_def["hvcs_client_name"],
+                        origin_url=cfg_mr_def.get("origin_url")
+                        or example_git_https_url,
+                        hvcs_domain=cfg_mr_def["hvcs_domain"],
+                    )
+                    commit_type = cfg_mr_def["commit_type"]
+
+                elif action == RepoActionStep.CONFIGURE_MONOREPO:
+                    cfg_mr_pkg_def = cast(
+                        "RepoActionConfigureMonorepoDetails", step_result["details"]
+                    )
+                    configure_monorepo_package(
+                        dest_dir=cfg_mr_pkg_def["package_dir"],
+                        commit_type=commit_type,
+                        hvcs_client_name=hvcs.__class__.__name__.lower(),
+                        hvcs_domain=str(hvcs.hvcs_domain),
+                        tag_format_str=cfg_mr_pkg_def["tag_format_str"],
+                        extra_configs=cfg_mr_pkg_def["extra_configs"],
+                        mask_initial_release=cfg_mr_pkg_def["mask_initial_release"],
+                        package_name=cfg_mr_pkg_def["package_name"],
+                        monorepo=True,
+                    )
+
+                elif action == RepoActionStep.CHANGE_DIRECTORY:
+                    change_dir_def = cast(
+                        "RepoActionChangeDirectoryDetails", step_result["details"]
+                    )
+                    if not (
+                        new_cwd := Path(change_dir_def["directory"])
+                        .resolve()
+                        .absolute()
+                    ).exists():
+                        msg = f"Directory {change_dir_def['directory']} does not exist."
+                        raise NotADirectoryError(msg)
+
+                    # Helpful Transform to find the project root repo without needing to pass it around (ie '/' => repo_dir)
+                    new_cwd = (
+                        repo_dir if str(new_cwd) == str(repo_dir.root) else new_cwd
+                    )
+
+                    if not new_cwd.is_dir():
+                        msg = f"Path {change_dir_def['directory']} is not a directory."
+                        raise NotADirectoryError(msg)
+
+                    # TODO: 3.9+, use is_relative_to
+                    # if not new_cwd.is_relative_to(repo_dir):
+                    if repo_dir != new_cwd and repo_dir not in new_cwd.parents:
+                        msg = f"Cannot change directory to '{new_cwd}' as it is outside the repo directory '{repo_dir}'."
+                        raise ValueError(msg)
+
+                    os.chdir(str(new_cwd))
+
                 elif action == RepoActionStep.MAKE_COMMITS:
                     mk_cmts_def = cast(
                         "RepoActionMakeCommitsDetails", step_result["details"]
@@ -1528,6 +1719,18 @@ def build_repo_from_definition(  # noqa: C901, its required and its just test co
                                 create_branch_def["name"],
                                 commit=start_head.commit,
                             )
+                            # set up a remote tracking branch for the new branch
+                            with git_repo.config_writer("repository") as config:
+                                config.set_value(
+                                    f'branch "{create_branch_def["name"]}"',
+                                    "remote",
+                                    "origin",
+                                )
+                                config.set_value(
+                                    f'branch "{create_branch_def["name"]}"',
+                                    "merge",
+                                    f'refs/heads/{create_branch_def["name"]}',
+                                )
                             new_branch_head.checkout()
 
                         elif "branch" in ckout_def:
@@ -1571,19 +1774,19 @@ def build_repo_from_definition(  # noqa: C901, its required and its just test co
                         )
 
                 elif action == RepoActionStep.GIT_MERGE:
-                    this_step = cast("RepoActionGitMerge", step_result)
+                    this_step = cast(
+                        "RepoActionGitMerge[RepoActionGitFFMergeDetails | RepoActionGitMergeDetails]",
+                        step_result,
+                    )
 
                     with Repo(repo_dir) as git_repo:
                         if this_step["details"]["fast_forward"]:
-                            ff_merge_def = cast(
-                                "RepoActionGitFFMergeDetails", this_step["details"]
+                            git_repo.git.merge(
+                                this_step["details"]["branch_name"], ff=True
                             )
-                            git_repo.git.merge(ff_merge_def["branch_name"], ff=True)
 
                         else:
-                            merge_def = cast(
-                                "RepoActionGitMergeDetails", this_step["details"]
-                            )
+                            merge_def = this_step["details"]
 
                             # Update the commit definition with the repo hash
                             merge_def["commit_def"] = create_merge_commit(
@@ -1609,6 +1812,15 @@ def build_repo_from_definition(  # noqa: C901, its required and its just test co
                                 }
                             )
 
+                        # After a merge we need to ensure the remote tracking branch is updated
+                        with suppress(TypeError):
+                            # Fake an automated push to remote by updating the remote tracking branch
+                            # Detached HEAD commits won't have an active branch and throw a TypeError
+                            git_repo.git.update_ref(
+                                f"refs/remotes/origin/{git_repo.active_branch.name}",
+                                git_repo.head.commit.hexsha,
+                            )
+
                 else:
                     raise ValueError(f"Unknown action: {action}")
 
@@ -1735,7 +1947,14 @@ def split_repo_actions_by_release_tags(
 
         # Loop through all actions and split them by release tags
         for step in repo_definition:
-            if any(step["action"] == action for action in [RepoActionStep.CONFIGURE]):
+            if any(
+                step["action"] == action
+                for action in [
+                    RepoActionStep.CONFIGURE,
+                    RepoActionStep.CREATE_MONOREPO,
+                    RepoActionStep.CONFIGURE_MONOREPO,
+                ]
+            ):
                 releasetags_2_steps[None].append(step)
                 continue
 
@@ -1754,6 +1973,7 @@ def split_repo_actions_by_release_tags(
 
         insignificant_actions = [
             RepoActionStep.GIT_CHECKOUT,
+            RepoActionStep.CHANGE_DIRECTORY,
         ]
 
         # Remove Unreleased if there are no significant steps in an Unreleased section
